/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| Copyright IBM Corporation 2012-2013.  All Rights Reserved.             |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
	// summary:
	//	Implement a resource picker for Impact Analysis
	//
	// description:
	//	Uses RMPSPicker (simple search on DM repository) as a picker.
	//	Note: This module is loaded by the com.ibm.xtools.rmps.web.app.webUiInitializers extension point
	//	so that it can install the picker functionality for IA. See com.ibm.xtools.impactanalysis.rmps.host.web/plugin.xml

	dojo.provide('impactanalysis.app.rmps.SimpleIAResourceProvider');

	dojo.require("dojo.i18n");

	dojo.require('dm.analysis.app.ui.picker');
	dojo.require("dm.analysis.app.ui.internal.impactanalysis.picker.IASearchBox");
	dojo.require('jazz.app.context');
	dojo.require('jazz.ui.internal.ResourceLinkAggregator');

	dojo.require('rmps.app.ui.internal.dialogs.SelectCALMResourcesDialog');
	dojo.require("rmps.app.ui.internal.util.CLMUtil");
	dojo.require("rmps.app.core.internal.query.TextQuery");
	dojo.require("rmps.core.streams.StreamClient");
	dojo.require("rmps.core.query.QueryClient");

	dojo.requireLocalization("impactanalysis.app.rmps", "Messages");

	var messages = dojo.i18n.getLocalization("impactanalysis.app.rmps", "Messages"),
		pickerFunction;

	var CLMUtil = rmps.app.ui.internal.util.CLMUtil;
	var StreamClient = rmps.core.streams.StreamClient;
	var QueryClient = rmps.core.query.QueryClient;

	var _linkAggregator = jazz.ui.internal.resourceLinkAggreagtor();

	pickerFunction = function(callbackFunction) {
		// summary:
		//		the function that opens the RMPS picker for IA
		// description:
		//		opens a DM resource picker (RMPSPicker), and when a resource is selected
		//		informs IA by calling the passed callback function with the resource URI
		var dialog;

		var onOKCallback = dojo.hitch(this, function(resources) {
			if (resources && resources.length > 0) {
				var resourceURIs = dojo.map(resources, function(resource) {
					return resource["url"];
				});
				callbackFunction(resourceURIs[0]);
			}
		});

		var onCloseCallback = dojo.hitch(this, function() {
			dialog.destroyRecursive();
			delete dialog;
		});

		var params = {
			dialogTitle: messages.pickerPrimaryTitle,
			dialogSecondaryTitle: null,
			resourceUri: null,
			onOk: onOKCallback,
			onClose: onCloseCallback
		};

		/*Open the dialog */
		dialog = new rmps.app.ui.internal.dialogs.SelectCALMResourcesDialog(params);
	};

	// register the picker function with IA
	dm.analysis.app.ui.picker.setPicker(pickerFunction);

	var iaSearchBoxPrototype = dm.analysis.app.ui.internal.impactanalysis.picker.IASearchBox.prototype;

	_formatResults = function(searchResults) {
		var formattedResults = [];
		var entries = searchResults.entries;

		for (var i = 0; i < entries.length; i++) {
			var entry = entries[i];

			var blankIcon = dojo.moduleUrl("dojo", "resources/blank.gif");
			var uri = entry["rdf:Description"] && entry["rdf:Description"][0] ? entry["rdf:Description"][0]["rdf:about"] : null;
			var text = entry.title[0].value;

			_linkAggregator.register(uri, dojo.hitch(this, _richPreviewDidLoad, uri), null );

			var label = '<span class="IASearchBox-search-result">' +
				'<img height="16" width="16" src="' + blankIcon + '" id="' + encodeURIComponent(uri) + '">' + text + 
			'<span>';

			var singleResult = {
				label: label,
				handler: this.getHandler(uri, label)
			};

			formattedResults.push(singleResult);
		}
		return formattedResults;
	};

	_richPreviewDidLoad = function(uri, result){
		var imgNode = dojo.query('#' + encodeURIComponent(uri));
		imgNode = imgNode.length === 1 ? imgNode[0] : null;

		if (imgNode){
			var icon;
			if (result.oslcCompact) {
				icon = result.oslcCompact.icon;
			} else if (result.compactRendering) {
				icon = result.compactRendering.icon;
			} else {
				console.debug('Unknown preview format:', result); //$NON-NLS-1$
				return;
			}	

			dojo.attr(imgNode, 'src', icon);
		}
	};

	_getFirstDMProjectLink = function(projectLinks) {
		var dmProjectLink;
		dojo.every(projectLinks, function(project) {
			if (project.serviceProviders && project.serviceProviders[0] && project.serviceProviders[0].isImplicit) {
				var splitArray = project.serviceProviders[0].details.path.split('/');
				dmProjectLink = splitArray[splitArray.length - 1];
				return false;
			}
			return true;
		});
		return dmProjectLink;
	};

	performSearch = function(query, projectId, baseline, callback, errback) {
		var textQuery = new rmps.app.core.internal.query.TextQuery({
			includeSuppliers: "false",
			includeSystemDomains: "false",
			limit: "25",
			pageSize: "25",
			onlyType: "http://jazz.net/ns/dm/core#Document",
			projectId: projectId,
			q: query,
			resourcesOnly: "false",
			"rmps.context": baseline,
			sort: ""
		});
		var results = null;
		var searchDeferred = QueryClient.execute(textQuery, results);

		searchDeferred.addCallback(this, callback);
		searchDeferred.addErrback(this, errback);
	};

	performUpdate = function(query, scope, callback, errorCallback) {
		if (this._previousQuery != query) {
			this._previousQuery = query;
		}

		var self = this;

		if (this._prevXhr) {
			this._prevXhr.cancel();
		}

		CLMUtil.getProjectLinks(null, dojo.hitch(this, function(projectLinks) {
			if (projectLinks.length > 0) {
				var dmProjectLink = _getFirstDMProjectLink(projectLinks);
				var projectContext = null;

				dojo.when(StreamClient.getDefaultStream(dmProjectLink), dojo.hitch(this, function(contextURI) {
					projectContext = contextURI && contextURI.length > 0 ? contextURI : null;
					performSearch(query, dmProjectLink, projectContext, 
						function(data) {
							var queryResults = dojo.hitch(self, _formatResults)(data);
							callback(queryResults);
						}, 
						function(e) {
							debugger;
							e && errorCallback(e.message);
						});
				}));

			} else {
				CLMUtil.showNoProjectLinksFoundContentDialog();
				PlatformUI.getWorkbench().hideMessageArea();
			}
		}));

		return;

		//TODO: what is this handle callback?
		/*handle: function() {
			self._prevXhr = null;
		}*/
	};

	iaSearchBoxPrototype.performUpdate = function(query, scope, callback, errorCallback) {
		dojo.hitch(this, performUpdate)(query, scope, callback, errorCallback);
	};

}());