/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
/*jslint nomen: true, sloppy: true, indent: 4, maxerr: 50 */
(function () {
	dojo.provide("dm.analysis.app.ui.DiagramUtilityMethods");
	
	/** This utility class provides hooks for calling either DM or RELM versions
	 *  of functions for various diagram logic. For every method, the pattern is
	 *  the same: 
	 *
	 *  1) There is a public function called set_<MethodName>(specificFunction)
	 *
	 *  2) There is a public function called call_<MethodName>(args)
	 *
	 *  3) Each application (DM or RELM) creates an initializer class which 
	 *     defines its version of the function and calls set_<MethodName>
	 *     with that application specific function.
	 *		<code>
	 *		(function () {
	 *			dojo.provide('my.analysis.utility.methods');
	 *			dojo.require('dm.analysis.app.ui.DiagramUtilityMethods');
	 *
	 *			var myFunction = function(resourceUri) {
	 *				var resultUri = null; // to be returned by this function
	 *				// logic to compute the result URI based on the resource Uri
	 *				return resultUri;
	 *			}
	 *
	 *			dm.analysis.app.ui.DiagramUtilityMethods.set_Function(myFunction);
	 *		}());
	 *		</code>
	 *
	 *  4) The application must define a plug-in 'webUiInitializer' extension to load the initializer class.
	 *		<code>
	 *			<extension point="com.ibm.xtools.rmps.web.app.webUiInitializers">
	 *				<initializer jsModule="my.analysis.utility.methods" />
	 *			</extension>
	 *		</code>
	 *
	 *  5) The common analysis code will then call the call_<MethodName>
	 *     function when needed.
	 *
	 *  The following methods are required by each application:
	 *
	 *      getCompactRenderingUri( resourceUri );
	 *			This method takes a resource URI (from an analysis node) and returns
	 *			a URI that can be used to perform the compact rendering of that resource.
	 *
	 *		getResourceEditUri( resourceUri );
	 *			This method takes a resource URI (from an analysis node) and returns
	 *			a URI that can be used to open an editor for that resource.
	 */

	// Define the application specific functions that can be called.
	var getCompactRenderingUri = null;
	var getResourceEditUri = null;
	var fullTraceEnabled = false;

	dm.analysis.app.ui.DiagramUtilityMethods = {

		/** renderingUri = getCompactRenderingUri(resourceUri); */
		
		set_getCompactRenderingUri: function (appFunction) {
			if (dojo.isFunction(appFunction)) {
				getCompactRenderingUri = appFunction;
			}
		},

		call_getCompactRenderingUri: function (resourceUri) {
			if (getCompactRenderingUri) {
				return getCompactRenderingUri(resourceUri);
			}
			return null;
		},
		


		/** resourceEditUri = getResourceEditUri(resourceUri); */

		set_getResourceEditUri: function (appFunction) {
			if (dojo.isFunction(appFunction)) {
				getResourceEditUri = appFunction;
			}
		},

		call_getResourceEditUri: function (resourceUri) {
			if (getResourceEditUri) {
				return getResourceEditUri(resourceUri);
			}
			return null;
		},
		
		enableFullTrace: function(turnOn) {
			fullTraceEnabled = turnOn;
		},
		
		isFullTraceEnabled: function() {
			return fullTraceEnabled;
		}
	};

}());