/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function () {
	dojo.provide('dm.analysis.app.ui.internal.diagram.LongRunningWorkSequencer');
	
	dojo.require('dojo.string');
	dojo.require('dojox.lang.functional');
	
	var df = dojox.lang.functional;
	
	dojo.declare('dm.analysis.app.ui.internal.diagram.LongRunningWorkSequencer', [], {
		// summary:
		//		A sequencer of sequential work tasks that are long running.
		// description:
		//		Defines a sequence of small tasks that implement a long running task. The
		//		sequencer runs these tasks in order, while allowing the Javascript engine
		//		a pause after each work increment so that the browser remains responsive.
		
		_workArray: null,
		_deferred: null,
		
		constructor: function () {
			this._workArray = [];
			this._deferred = new dojo.Deferred(dojo.hitch(this, "_handleCancel"));
		},
		
		step: function (name, workFunction) {
			// summary:
			//		Define a step in the long running sequence.
			// description:
			//		Define a step in the long running sequence. The name is a value suitable for display
			//		in progress messages. The work function may optionally return true to indicate that
			// 		the step is incomplete, in which case the function will be rescheduled.
			//		This method returns the LongRunningWorkSequencer instance so that steps can be easily chained.
			this._workArray.push({name: name, work: workFunction});
			return this;
		},
		
		schedule: function (data) {
			// summary:
			//		Start the work sequence, returning a dojo.Deferred to track progress
			
			var status;
			
			status = {
				workPhase: 0,
				data: data,
				step: [],
				done: false
			};
			
			this._scheduleNextWorkIncrement(status);
			
			return this.getPromise();
		},
		
		getPromise: function () {
			return this._deferred.promise;
		},
		
		_handleCancel: function () {
			return dojo.mixin(new Error(), { cancelled: true });
		},

		_scheduleNextWorkIncrement: function (s) {
			// summary:
			//		Schedule the next work increment
			if(this._deferred.fired > 0) {
				// already fired, either because of completion or an error
				return;
			}
			var workFunction = this._computeWorkIncrementFunction(s),
				d = this._deferred;
			if (workFunction) {
				setTimeout(workFunction, 20);
			} else {
				// no work function, callback with with the work status
				d.callback(s.workPhase === this._workArray.length);
			}			
		},
		
		_computeWorkIncrementFunction: function (s) {
			var i = s.workPhase,
				workSteps = this._workArray.length,
				workFunction,
				stepName,
				safeWorkFunction,
				d = this._deferred,
				step;
				
			if (i < workSteps) {
				workFunction = this._workArray[i].work;
				stepName = this._workArray[i].name;
			}
			
			step = s.step[i] = s.step[i] || { name: stepName, step: i };
			
			
			if (workFunction && dojo.isFunction(workFunction)) {
				safeWorkFunction = dojo.hitch(this, function () {
					var stepIncomplete = false,
						start,
						time,
						progressObject;
					try {
						// do the work
						start = new Date().getTime();
						stepIncomplete = workFunction(step, s.data);
						time = new Date().getTime() - start;
						
						// prepare and schedule the next iteration
						if (!stepIncomplete) {
							s.workPhase += 1;
						}
						this._scheduleNextWorkIncrement(s);
						
						// report progress
						progressObject = {
							message: dojo.string.substitute(step.name, [step.iterator ? step.iterator.getProgressMessage() : '']),
							time: time,
							stepIncomplete: stepIncomplete
						};
						d.progress(progressObject);
					} catch (e) {
						// let the deferred object know about the error.
						d.reject(e);
					}
				});
			}
			return safeWorkFunction;
		}
		
	});
	
}());